--[[ LibDrag
  Library.LibDrag.draggify(window)
]]--

if not Library then Library = {} end
local Drag = {}
if not Library.LibDrag then Library.LibDrag = Drag end

Drag.DebugLevel = 0
Drag.Version = "0.5-130712-20:34:32"

Drag.frames = {}

function Drag.draggify(frame, callback)
  local newtab = { dragging = false, x = 0, y = 0 }
  Drag.frames[frame] = newtab
  newtab.callback = callback
  frame:EventAttach(Event.UI.Input.Mouse.Left.Down, function(...) Drag.leftdown(frame, ...) end, "Drag_left_down")
  frame:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(...) Drag.mousemove(frame, ...) end, "Drag_mouse_move")
  frame:EventAttach(Event.UI.Input.Mouse.Left.Up, function(...) Drag.leftup(frame, ...) end, "Drag_left_up")
  frame:EventAttach(Event.UI.Input.Mouse.Left.Upoutside, function(...) Drag.leftupoutside(frame, ...) end, "Drag_left_upoutside")
  Drag.frames[frame] = newtab
end

function Drag.leftdown(frame, event, ...)
  local win = Drag.frames[frame]
  if not win then
    Drag.frames[frame] = { dragging = false, x = 0, y = 0 }
    win = Drag.frames[frame]
  end
  win.dragging = true
  win.win_x = frame:GetLeft()
  win.win_y = frame:GetTop()
  local l, t, r, b = frame:GetBounds()
  frame:ClearAll()
  frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", l, t)
  frame:SetWidth(r - l)
  frame:SetHeight(b - t)
  win.ev_x = Inspect.Mouse().x
  win.ev_y = Inspect.Mouse().y
end

function Drag.mousemove(frame, event, ...)
  local event, x, y = ...
  local win = Drag.frames[frame]
  if win and win.dragging then
    local win = Drag.frames[frame]
    local new_x = win.win_x + x - win.ev_x
    local new_y = win.win_y + y - win.ev_y
    frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", new_x, new_y)
    if win.callback then
      win.callback(frame, new_x, new_y)
    end
  end
end

function Drag.leftup(frame, event, ...)
  local win = Drag.frames[frame]
  if win then
    win.dragging = false
    win.ev_x = nil
    win.ev_y = nil
  end
end

function Drag.leftupoutside(frame, event, ...)
  local win = Drag.windows[frame]
  if win then
    win.dragging = false
    win.ev_x = nil
    win.ev_y = nil
  end
end
